# -*- coding: utf-8 -*-
"""
Created on Tue Jul 23 11:03:47 2019

@author: Man Vinayaka
"""


def linearRelation(df, pol1, pol2):
    """ This plot given two pollutants will graph the linear 
        relationship. The df given will be converted to the slope
        between the pollutants. 
		
		Parameters
		----------
		df: data frame
			minimally containing date and two pollutants
		pol1: type string
			First pollutant that when plotted would appear
			on the x-axis of a relationship, ex: 'nox'
		pol2: type string
			Second pollutant that when plotted would appear
			on the y-axis of a relationship, ex: 'pm10'		
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import seaborn as sns

    df.index = pd.to_datetime(df.date)
    unique_years = np.unique(df.index.year)

    """ Finds the unique years in the given data
        and converts it to a string type within a list 
        to be called on later to determine manipulation 
        of data
    """
    i = 0
    year = []
    while i < len(unique_years):
        year.append(str(unique_years[i]))
        i = i + 1
    # print(year)
    num_unique_years = len(year)
    df = df.drop("date", axis=1)

    """ Will create a df(df_1) varying in lentgh based on the
        number of years that the data contains. The df 
        consists of the slope between the two given pollutants
    """
    i = 0
    values = []
    while i < num_unique_years:
        df_new = df[year[i]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        # df_new.index.dayofweek
        nox = df_new[pol1].mean()
        so2 = df_new[pol2].mean()
        values.append(so2 / nox)
        i = i + 1
    df_1 = pd.DataFrame(values)
    df_1.columns = ["div"]
    # print(df_1)
    """ Creats a df for the first polutant that 
        contains thea average value for every month
        of every year given
    """
    var1 = []
    var2 = []
    i = 0
    x = 0
    j = 0
    while i < num_unique_years:

        df_new = df[year[j]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        df_new["day"] = df_new.index.dayofweek
        df_new["hour"] = df_new.index.hour
        i = i + 1
        j = j + 1
        x = 0
        while x < 12:
            a = df_new[df_new.month == x]
            mean_var1 = a[pol1].mean()
            var1.append(mean_var1)
            x = x + 1

    """ Creats a df for the second polutant that 
        contains thea average value for every month
        of every year given
    """
    i = 0
    x = 0
    j = 0
    while i < num_unique_years:

        df_new = df[year[j]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        df_new["day"] = df_new.index.dayofweek
        df_new["hour"] = df_new.index.hour
        i = i + 1
        j = j + 1
        x = 0
        while x < 12:
            a = df_new[df_new.month == x]
            mean_var2 = a[pol2].mean()
            var2.append(mean_var2)
            x = x + 1
    """ Finds the slope of the new dataframe
    """
    res = [i / j for i, j in zip(var2, var1)]
    df_2 = pd.DataFrame(res)
    df_2.columns = [pol2 + "/" + pol1]

    """ creates a list that contains the year values 
        of each data point to use for graphing a 
        scatterplot
    """
    scatterY = []
    t = 0
    while t < num_unique_years:
        r = 0
        while r < 12:
            scatterY.append(t + (r / 12))
            r = r + 1
        t = t + 1

    """ Plots a line plot of the average slope between the
        two pollutants given for every year. That plot is then superimposed 
        with a scatter plot of the average value of every month of every year
    """
    fig = plt.figure()
    ax = fig.add_subplot(111, label="1")
    ax2 = fig.add_subplot(111, label="2", frame_on=False)

    ax.set_ylim(bottom=0.015, top=0.040)
    ax.plot(df_1, color="red")
    ax.set_xlabel("Year", color="red")
    ax.set_ylabel(pol2 + "/" + pol1, color="red")
    ax.tick_params(axis="x", colors="red")
    ax.tick_params(axis="y", colors="red")

    ax2.scatter(scatterY, res, color="blue")
    ax2.xaxis.tick_top()
    ax2.yaxis.tick_right()
    ax2.xaxis.set_label_position("top")
    ax2.yaxis.set_label_position("right")
    ax2.tick_params(axis="x", colors="blue")
    ax2.tick_params(axis="y", colors="blue")

    plt.show()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# linearRelation(df,'nox','so2')
# =============================================================================



def trendLevel(df, pollutant, **kwargs):
    """
    Plot that shows the overall pollutant trend for every year in the 
    df. It takes the average hour value of each month and plots a heatmap
    showing what timesof the year there is a high concentration of the 
    pollutant.
	
	Parameters
	----------
	df: data frame
		data frame of complete data
	pollutant: type string
		Name of the data series in df to produce plot
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    from numpy import vstack
    import pandas as pd
    from numpy import array
    import seaborn as sns

    df.index = pd.to_datetime(df.date)
    pollutant_series = df[pollutant]
    unique_years = np.unique(df.index.year)[:6]
    num_unique_years = len(unique_years)
    fig, ax = plt.subplots(nrows=num_unique_years, sharex=True, figsize=(10, 50))

    for i, unique_years in enumerate(unique_years):
        year_string = f"{unique_years}"
        pollutant_series_year = pollutant_series[year_string]
        t = pollutant_series_year.groupby(
            [pollutant_series_year.index.month, pollutant_series_year.index.hour]
        ).mean()
        two_d_array = t.values.reshape(12, 24).T
        sns.heatmap(
            two_d_array,
            cbar=True,
            linewidth=0,
            cmap="Spectral_r",
            vmin=0,
            vmax=400,
            ax=ax[i],
        )
        ax[i].set_title(year_string)
        ax[i].invert_yaxis()
    return ax


# =============================================================================
# df = pd.read_csv("mydata.csv")
# trendLevel(df,'nox')
# =============================================================================



# -- coding utf-8 --
"""
Created on Thu Jul 25 143303 2019

@author Man Vinayaka
"""


def interpolPlot(
    df, shape_df, long, lat, pollutant, 
    resolution=100, partitions=15, cmap="inferno",
    Tcolor = 'red', markersize = 3, plot_train_points=False
):

    import geopandas
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    from geopandas import GeoDataFrame
    from sklearn.ensemble import RandomForestRegressor
    from matplotlib.colors import ListedColormap
    from shapely.geometry import (Polygon, MultiPolygon, Point)

    from polire.custom import CustomInterpolator

    df = df.sample(frac=1)
    trainX = df[[long, lat]].values
    trainy = df[pollutant].values

    def collec_to_gdf(collec_poly):
        """Transform a `matplotlib.contour.QuadContourSet` to a GeoDataFrame"""
        polygons, colors = [], []
        for i, polygon in enumerate(collec_poly.collections):
            mpoly = []
            for path in polygon.get_paths():
                try:
                    path.should_simplify = False
                    poly = path.to_polygons()
                    # Each polygon should contain an exterior ring + maybe hole(s):
                    exterior, holes = [], []
                    if len(poly) > 0 and len(poly[0]) > 3:
                        # The first of the list is the exterior ring :
                        exterior = poly[0]
                        # Other(s) are hole(s):
                        if len(poly) > 1:
                            holes = [h for h in poly[1:] if len(h) > 3]
                    mpoly.append(Polygon(exterior, holes))
                except:
                    print('Warning: Geometry error when making polygon #{}'
                        .format(i))
            if len(mpoly) > 1:
                mpoly = MultiPolygon(mpoly)
                polygons.append(mpoly)
                colors.append(polygon.get_facecolor().tolist()[0])
            elif len(mpoly) == 1:
                polygons.append(mpoly[0])
                colors.append(polygon.get_facecolor().tolist()[0])
        if type(collec_poly.cmap) != ListedColormap:
            raise ValueError("""We only support ListedColormap right now.\n"""
                """simply convert your cmap to ListedColormap using url here."""
                """https://matplotlib.org/3.1.0/tutorials/colors/colormap-manipulation.html#sphx-glr-tutorials-colors-colormap-manipulation-py""")

        ixs = [collec_poly.cmap.colors.index(c[:3]) for c in colors]
        return GeoDataFrame(
            geometry=polygons,
            data={'RGBA': colors, 
                'cmapIX': ixs},
            crs={'init': 'epsg:4269'})

    z = trainy
    x1max, x2max = np.max(trainX, axis=0)
    x1min, x2min = np.min(trainX, axis=0)
    xi = np.linspace(x1min, x1max, resolution)
    yi = np.linspace(x2min, x2max, resolution)
    Xi, Yi = np.meshgrid(xi, yi)
    t = CustomInterpolator(RandomForestRegressor)
    t.fit(trainX, z)

    zi = t.predict(np.asarray([Xi.ravel(), Yi.ravel()]).T)
    zi = zi.reshape(Xi.shape)

    vmin = zi.min()
    vmax = zi.max()
    collec_poly = plt.contourf(
        Xi, Yi, zi, partitions, vmin=vmin, vmax=vmax, cmap=cmap
    )
    plt.close()

    gdf = collec_to_gdf(collec_poly)
    # new min max vales in term of index of colors
    vmax2 = gdf.cmapIX.max()
    vmin2 = gdf.cmapIX.min()

    # intersection with shape_df
    inter = geopandas.overlay(shape_df, gdf, how='intersection')
    ax = inter.plot(
        column = 'cmapIX',
        cmap=cmap,
        figsize=(40, 40),
        vmax = vmax2,
        vmin = vmin2,
    )
    ax = shape_df.plot(
        ax = ax,
        color = 'none', 
        edgecolor='k',
        figsize=(40, 40)
    )
    # getting geodataframe of the train points
    if plot_train_points:
        geometry = [Point(xy) for xy in zip(df[long], df[lat])]
        geodf = geopandas.GeoDataFrame(
            df, crs={'init': 'epsg:4269'},
            geometry=geometry
        )

        # finding the intersection and plotting
        from geopandas.tools import sjoin
        inter2 = sjoin(geodf, shape_df)
        inter2.plot(
            ax = ax, color=Tcolor, label="Train points",
            markersize=markersize
        )
        
    plt.axis('off')
    fig = ax.get_figure()
    cax = fig.add_axes([0.9, 0.3, 0.03, 0.4])
    sm = plt.cm.ScalarMappable(
        cmap=cmap, 
        norm=plt.Normalize(vmin=vmin, vmax=vmax)
        )
    sm._A = []
    ax.legend()

    bounds = np.linspace(vmin, vmax, partitions)
    fig.colorbar(
        sm, cax=cax, ticks=bounds,
        boundaries=bounds, format='%1.1E')
    return ax


# =============================================================================
# from geopandas import GeoDataFrame
# import pandas as pd
# import matplotlib.pyplot as plt
# from vayu import interpolPlot

# shapeFile = "re/tl_2017_us_state.shp"
# long = "Longitude"
# lat = "Latitude"
# pollutant = "Arithmetic Mean"
# df = pd.read_csv("re/daily_44201_2018.csv")
# shape_df = GeoDataFrame.from_file(shapeFile)
# shape_df.drop(shape_df.index[[34, 35, 36, 40, 41, 49, 31]], inplace=True)
# interpolPlot(df, shape_df, long, lat, pollutant)
# plt.show()
# =============================================================================



# -*- coding: utf-8 -*-
"""
Created on Sun Jul 21 19:40:26 2019

@author: Man Vinayaka
"""

# Select by Data
def selectByDate(df, year):
    """ Utility function to cut given dataframe by the year 
        and find the average value of each day 
		
		Parameters
		----------
		df: data frame
			a data frame containing a date field
		year: type string
			a year to select to cut data
    """
    import pandas as pd
    import numpy as np

    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_n = df[year].resample("1D").mean()
    df_n = df_n.fillna(method="ffill")
    df_n["month"] = df_n.index.month
    df_n.index.dayofweek
    print(df_n)


# =============================================================================
# df = pd.read_csv("mydata.csv")
# selectByDate(df,'2003')
# =============================================================================



# -*- coding: utf-8 -*-
"""
Created on Mon Jul 22 12:48:26 2019

@author: Man Vinayaka
"""


def summaryPlot(df):
    """ Plots import summary of data frame given. Plots line plots
        and histograms for each polutant as well as statiscs such as 
        mean,max,min,median, and 95th percentile
		
		Parameters
		----------
		df: data frame
			data frame to be summarised. Must contain a date field
			and at least one other parameter 
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import matplotlib.patches as mpatches
    import seaborn as sns
    from matplotlib.pyplot import figure

    class color:
        # Allows for bolded and underlined text
        BOLD = "\033[1m"
        UNDERLINE = "\033[4m"
        END = "\033[0m"

    # Reads df and fills empty values
    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_all = df.resample("1D")
    df_all = df_all.fillna(method="ffill")

    dataPoints = ["pm25", "co", "so2", "pm10", "o3", "no2", "nox", "wd", "ws"]

    i = 0
    sub = 1
    while i < 9:
        # Plots line and histogram plots for ecery polutant
        # in the correct location based on subplot
        plt.figure(1, figsize=(50, 50))
        plt.subplot(9, 2, sub)
        sub = sub + 1
        a = df_all[dataPoints[i]].plot.line(color="gold")
        a.axes.get_xaxis().set_visible(False)
        a.yaxis.set_label_position("left")
        plt.ylabel(dataPoints[i], fontsize=75, bbox=dict(facecolor="whitesmoke"))
        # print(df['pm25'].max())

        plt.subplot(9, 2, sub)
        sub = sub + 1
        plt.hist(df_all[dataPoints[i]], bins=50, color="green")
        i = i + 1
    i = 0
    while i < 9:
        # Calculates statistics
        nDf = df[dataPoints[i]]
        missing = nDf.isna().sum() + sum(n < 0 for n in nDf)
        minVal = nDf.min()
        maxVal = nDf.max()
        meanVal = nDf.mean()
        medianVal = nDf.median()
        percentile = nDf.quantile(0.95)
        print("---------------")
        print(color.BOLD + color.UNDERLINE + dataPoints[i] + color.END)
        print("min = " + str(0))
        print("max = " + str(maxVal))
        print("missing = " + str(missing))
        print("mean = " + str(meanVal))
        print("median = " + str(medianVal))
        print("95th percentile = " + str(percentile))
        i = i + 1


# =============================================================================
# df = pd.read_csv('mydata.csv')
# summaryPlot(df)
# =============================================================================



def calendarPlot(df, pollutant, year, country, **kwargs):
    """ Plots a heatmap on a calendar layout based 
        on the intensity of the pollutant per day.
        Each day contains an arrow indicating both 
        wind direction as well as wind speed
		
		Parameters
		----------
		df: data frame
			minimally containing date and at least one other
			numeric variable 
		pollutant: type string
			A pollutant name correspoinding to 
			a variable in a data frame, ex: 'pm25'
		year: type string
			Year to plot', ex '2003'    
    cvals: numpy.ndarray
       array containing bin edges for the pollutant
    colors: numpy.ndarray
       array containing hex color codes corresponding to 
       bin edges for the pollutant  
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    from .utils import Legend

    cvals  = Legend.country_pollutants[country][pollutant]['bin_edges']
    colors = Legend.country_pollutants[country][pollutant]['color_scale']

    norm = plt.Normalize(min(cvals),max(cvals))
    tuples = list(zip(map(norm,cvals), colors))
    cmap = mpl.colors.LinearSegmentedColormap.from_list("", tuples)

    def calendar_array(dates, data):
        """ creates the calendar array returning i,j giving
            positional values in the array
        """        
        i, j = zip(*[d.isocalendar()[1:] for d in dates])        
        
        i = np.array(i) - min(i)
        j = np.array(j) - 1
        ni = max(i) + 1

        calendar = np.nan * np.zeros((ni, 7))
        calendar[i, j] = data
        return i, j, calendar

    def calendar_heatmap(ax, dates, data):
        """ Sets heatmap information
        """
        i, j, calendar = calendar_array(dates, data)
        im = ax.imshow(calendar, interpolation="none", cmap= cmap)
        label_days(ax, dates, i, j, calendar)

    def label_days(ax, dates, i, j, calendar):
        """ Based on the day of the week, it will print that 
            text on each box of the day. The arrow is also ploted on
            each box for every day converting the given wind direction
            to a xy coordinate
        """
        ni, nj = calendar.shape
        day_of_month = np.nan * np.zeros((ni, 7))
        day_of_month[i, j] = [d.day for d in dates]

        for (i, j), day in np.ndenumerate(day_of_month):
            if np.isfinite(day):
                ax.arrow(
                    j,
                    i,
                    avg_ws[int(day) - 1 + a]
                    * np.cos(avg_wd[int(day) - 1 + a] * np.pi / 180.0)
                    / 15.0,
                    -avg_ws[int(day) - 1 + a]
                    * np.sin(avg_wd[int(day) - 1 + a] * np.pi / 180.0)
                    / 15.0,
                    head_width=0.15,
                    head_length=0.1,
                    fc="k",
                    ec="k",
                )

        ax.set_yticklabels([])
        ax.set_xticklabels([])

    # =============================================================================
    #     Cuts given data to show average of each day.
    #     Adds a month coloumn to the df as well
    # =============================================================================

    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_year = df[year].resample("1D").mean()
    df_year["month"] = df_year.index.month
    df_year.index.dayofweek

    t = 1

    fig, ax = plt.subplots(figsize=(10, 10), nrows=4, ncols=4)

    # =============================================================================
    #     """ Plots 12 seperate plots that are then put togeather in a
    #         4x4 arrangement with the last column being used to plot
    #         the colorbar
    #     """
    # =============================================================================
    while t <= 12:

        avg_ws = []
        avg_wd = []
        avg_pm25 = []
        df_year_1 = df_year[df_year.month == t]
        avg_wd = df_year_1["wd"]
        avg_ws = df_year_1["ws"]
        avg_pm25 = df_year_1[pollutant]

        i = 1
        a = 0
        b = len(avg_pm25)
        while i <= 1:
            data = avg_pm25[a:b]
            num = len(data)
            if t == 12:
                start = dt.datetime(int(year), 1, 1)
            else:
                start = dt.datetime(int(year), t, 1)
            dates = [start + dt.timedelta(days=i) for i in range(num)]

            month_labels = [
                "               Jan               ",
                "               Feb               ",
                "               Mar               ",
                "               Apr               ",
                "               May               ",
                "               Jun               ",
                "               Jul               ",
                "               Aug               ",
                "               Sep               ",
                "               Oct               ",
                "               Nov               ",
                "               Dec               ",
            ]
            ax[(t - 1) // 3][(t - 1) % 3].set_title(
                month_labels[t - 1], bbox=dict(facecolor="whitesmoke")
            )
            calendar_heatmap(ax[(t - 1) // 3][(t - 1) % 3], dates, data)
            i = i + 1
        t = t + 1

    plt.tight_layout()

    # =============================================================================
    #    Colorbar plotting
    # =============================================================================
    grid = plt.GridSpec(4, 4, wspace=2, hspace=0.3)
    
    cbar_ax = plt.subplot(grid[:, 3])
    cb1 = mpl.colorbar.ColorbarBase(
        cbar_ax, cmap=cmap, norm=norm, orientation="vertical"
    )
    cb1.ax.tick_params(labelsize=15)

    plt.show()
    plt.close("all")


class Legend:
  import numpy as np
  import branca.colormap as cm

  country_pollutants = {
      'Canada' : {
          'AQHI' : {
            'notation' : 'Air Quality Health Index',
            'bin_edges' : np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 20]),
            'color_scale' : np.array(['#00ccff', '#0099cc', '#006699', '#ffff00','#ffcc00','#ff9933', 
                                       '#ff6666', '#ff0000', '#cc0000', '#990000', '#660000'])
          }
      },
      'Hong Kong' : {
          'AQHI' : {
            'notation' : 'Air Quality Health Index',
            'bin_edges' : np.array([3, 6, 7, 10, 20]),
            'color_scale' : np.array(['#008000', '#FFA500', '#FF0000', '#A52A2A', '#000000']) 
          }       
      },      
      'Mainland China' : {
          'AQI' : {
            'notation' : 'Air Quality Index',
            'bin_edges' : np.array([50,100, 150,200,300,450]),
            'color_scale' : np.array(['#00FF00', '#FFFF00','#FF9900','#FF0000','#540099','#800000'])
          }          
      },      
      'South Korea' : {
          'CAI' : {
            'notation' : 'Comprehensive Air-quality Index',
            'bin_edges' : np.array([50,100, 150,200,350,500]),
            'color_scale' : np.array(['#0000FF', '#005500','#FFFF00','#FF9900','#FF0000','#990000'])
          }          
      },      
      'United Kingdom' : {
          'APB' : {
            'notation' : 'Air Pollution Banding',
            'bin_edges' : np.array([1, 2, 3, 4, 5, 6, 7, 8, 9,10]),
            'color_scale' : np.array(['#CCFFCC', '#66FF66','#00FF00','#99FF00', '#FFFF00', '#FFCC00',
                                      '#FF6600', '#FF3300', '#FF0000', '#FF0066'])
          }          
      },
      'India' : {
          'AQI' : {
            'notation' : 'Air Quality Index',
            'bin_edges' : np.array([50, 100, 200, 300, 400, 500]),
            'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'so2': {      
            'notation' :'Sulphur dioxide',
            'bin_edges' : np.array([40, 80, 380, 800, 1600]),
            'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])      
          },
          'pm10': {        
              'notation' :'Particulate matter < 10 µm',
              'bin_edges' : np.array([ 50, 100, 250, 350, 430]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'pm25':  {
              'notation' :'Particulate matter < 2.5 µm',
              'bin_edges' : np.array([30, 60, 90, 120, 250]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'o3': {
              'notation' :'Ozone',
              'bin_edges' : np.array([50, 100, 168, 208, 748]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'no2': {
              'notation' :'Nitrogen Dioxide',
              'bin_edges' : np.array([40, 80, 180, 280, 400]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'co': {
              'notation' :'Carbon Monoxide',
              'bin_edges' : np.array([1, 2, 10, 17, 34]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'nh3': {
              'notation' : 'Ammonia',
              'bin_edges' : np.array([200, 400, 800, 1200, 1800]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          },
          'pb': {
              'notation' :'Lead',
              'bin_edges' : np.array([0.5, 1, 2, 3, 3.5]),
              'color_scale' : np.array(['#00CC00', '#66CC00', '#FFFF00', '#FF9900', '#FF0000', '#A52A2A'])
          }

      }, 
      'Europe' : {
          'CAQI': {
            'notation' : 'Common Air Quality Index (CAQI)',
            'bin_edges' : np.array([25, 50, 75, 100, 200]),
            'color_scale' : np.array(['#79BC6A', '#BBCF4C', '#EEC20B', '#F29305', '#E8416F'])
          }
      },
      'USA' : {
          'AQI': {
          'notation' : 'Air Quality Index',
          'bin_edges' : np.array([50, 100, 150, 200, 300, 500]),
          'color_scale' : np.array(['#00E400', '#FFFF00', '#FF7E00', '#FF0000', '#99004C', '#7E0023'])
          }
      }
  }

  def pollutant_legend(country, pollutant):
    colormap = Legend.cm.LinearColormap(colors=Legend.country_pollutants[country][pollutant]['color_scale'], 
                                        vmin=0, vmax=Legend.country_pollutants[country][pollutant]['bin_edges'][-1], 
                                        index = Legend.country_pollutants[country][pollutant]['bin_edges'])
    return colormap

  def pollutant_color_coding(poll, country, pollutant):  
    bin_edges = Legend.country_pollutants[country][pollutant]['bin_edges']  
    idx = np.digitize(poll, bin_edges, right=True)
    return Legend.country_pollutants[country][pollutant]['color_scale'][idx]



# # -*- coding: utf-8 -*-
# from pkg_resources import get_distribution, DistributionNotFound

# try:
#     # Change here if project is renamed and does not equal the package name
#     dist_name = "vayu"
#     __version__ = get_distribution(dist_name).version
# except DistributionNotFound:
#     __version__ = "unknown"
# finally:
#     del get_distribution, DistributionNotFound

from .timeInteractiveScatterPlot import timeInteractiveScatterPlot
from .scatterPlot import scatterPlot
from .timePlot import timePlot
from .calendarPlot import calendarPlot
from .linearRelation import linearRelation
from .selectByDate import selectByDate
from .summaryPlot import summaryPlot
from .timeProp import timeProp
from .timeVariation import timeVariation
from .trendLevel import trendLevel
from .windRosePy import windRose
from .interpolPlot import interpolPlot
from .smoothTrend import smoothTrend
from .interactiveChoroplethPlot import interactiveChoroplethPlot



def timeProp(df, year, pollutant, avg_days, 
             date_time_col_name):
    """ Plot a stacked bar graph of all data in the df
        based on frequency of wind direction in compass
        directions. Takes the average of every 3 days
        in each bar. The hight of the bar is value of 
        the pollutant that 3 day period. The bars are 
        binned proportionaly based on the overall value of the 
        pollutant 
        
        Parameters
        ----------
        df: data frame
            data frame which has the fields of date and the pollutant
            to be graphed
        year: type string
            The year of which the data will be cut
        pollutant: type string
            The pollutant of which to plot
    avg_days: type integer
      number of days to take the average of (controlling 
      granularity of the X-axis)
    date_time_col_name: type string
      df column name containing date-time stamps  
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import matplotlib.patches as mpatches

    # =============================================================================
    #     Cuts data into the year specified and averages the
    #     values of each day
    # =============================================================================
    df.index = pd.to_datetime(df[date_time_col_name])
    df = df.drop(date_time_col_name, axis=1)
    df_year = df[year]
    df_year = df_year.fillna(method="ffill")
    df_year["month"] = df_year.index.month

    # New df containing only the values of the pollutant specified
    polArray = df[year].resample("1D").mean()
    polArray = df_year[pollutant]

    nA = []
    neA = []
    eA = []
    seA = []
    sA = []
    swA = []
    wA = []
    nwA = []
    polMeanS = 0
    polMeanE = avg_days
    dfStart = 0
    dfEnd = 24*avg_days  # 24*3 for 3 day average

    x = 0

    while x < int(365/avg_days):  # example: 365 days / 3 = 121 floored. Represents number of bars total
        n = 0
        ne = 0
        e = 0
        se = 0
        s = 0
        sw = 0
        w = 0
        nw = 0
        a = df_year[dfStart:dfEnd]
        b = a["wd"]

        i = 0
        while i < 24*avg_days:  # Bins the wd data into categories for stacked bar graph
            if b[i] > 348.75 or b[i] < 33.75:
                n = n + 1
            elif b[i] > 33.75 and b[i] < 78.75:
                ne = ne + 1
            elif b[i] > 78.75 and b[i] < 123.75:
                e = e + 1
            elif b[i] > 123.75 and b[i] < 168.75:
                se = se + 1
            elif b[i] > 168.75 and b[i] < 213.75:
                s = s + 1
            elif b[i] > 213.75 and b[i] < 258.75:
                sw = sw + 1
            elif b[i] > 258.75 and b[i] < 303.75:
                w = w + 1
            elif b[i] > 303.75 and b[i] < 348.75:
                nw = nw + 1

            i = i + 1
        # calculates the 3 day proportion mean of each polutant and stores
        # it in a new list
        n = (n / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        ne = (ne / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        e = (e / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        se = (se / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        s = (s / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        sw = (sw / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        w = (w / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())
        nw = (nw / 24*avg_days) * (polArray[polMeanS:polMeanE].mean())

        nA.append(n)
        neA.append(ne)
        eA.append(e)
        seA.append(se)
        sA.append(s)
        swA.append(sw)
        wA.append(s)
        nwA.append(nw)
        x = x + 1
        # Adds to start and end values to get through end of df
        polMeanS = polMeanS + avg_days
        polMeanE = polMeanE + avg_days
        dfStart = dfStart + 24*avg_days
        dfEnd = dfEnd + 24*avg_days

    #########################################

    # Plots the stacked bar graph with specific color represtations.
    # A legend is also plotted
    X = np.arange(int(365/avg_days))

    data = np.array([nA, neA, eA, seA, sA, swA, wA, nwA])

    color_list = ["red", "blue", "green", "purple", "orange", "yellow", "brown", "pink"]
    X = np.arange(data.shape[1])
    
    for i in range(data.shape[0]):
        plt.bar(
            X,
            data[i],
            bottom=np.sum(data[:i], axis=0),
            color=color_list[i % len(color_list)],
        )
    red_patch = mpatches.Patch(color="red", label="north")
    blue_patch = mpatches.Patch(color="blue", label="north east")
    green_patch = mpatches.Patch(color="green", label="east")
    purple_patch = mpatches.Patch(color="purple", label="south east")
    orange_patch = mpatches.Patch(color="orange", label="south")
    yellow_patch = mpatches.Patch(color="yellow", label="south west")
    brown_patch = mpatches.Patch(color="brown", label="west")
    pink_patch = mpatches.Patch(color="pink", label="north west")
    plt.legend(
        bbox_to_anchor=(1.05, 1),
        loc=2,
        borderaxespad=0.0,
        handles=[
            red_patch,
            blue_patch,
            green_patch,
            purple_patch,
            orange_patch,
            yellow_patch,
            brown_patch,
            pink_patch,
        ],
    )

    plt.xticks(
        np.arange(0, int(365/avg_days), 30/avg_days),
        (
            "Jan",
            "Feb",
            "Mar",
            "Apr",
            "May",
            "Jun",
            "Jul",
            "Aug",
            "Sep",
            "Oct",
            "Nov",
            "Dec",
        ),
    )
    plt.title("Wind Direction")
    plt.xlabel("Contribution weighted by Mean")
    plt.ylabel("Pollutant (µg/m³)")
    plt.rcParams["figure.figsize"]= 20,6
    plt.show()

# =============================================================================
# mydata = pd.read_csv('mydata.csv')
# timeProp(mydata,'2003','so2')
# =============================================================================



# -*- coding: utf-8 -*-
"""
Created on Mon Jul 22 20:50:49 2019

@author: Man Vinayaka
"""


def timeVariation(df, pollutant):
    """ 4 seperate plots are plotted:
        
        1) the average pollutant level per day by each hour for each day of the 
            week across all of the data
        2) the average pollutant level by each hour, across all data
        3) the average pollutant level by each month of the year for across data
        4) the average pollutant level per day of the week across all data
		
		Parameters
		----------
		df: data frame
			data frame of hourly data. 
			Must include a date field and at least one variable to plot
		pollutant: type string
			Name of variable to plot
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array

    dayWeek = [
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday",
        "Sunday",
    ]

    # Adds month,day, and hour columns in df for futher pandas manipulation

    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_new = df
    df_new = df_new.fillna(method="ffill")
    df_new["month"] = df_new.index.month
    df_new["day"] = df_new.index.dayofweek
    df_new["hour"] = df_new.index.hour

    x = 0
    sub = 1
    # for plot 1
    while x < 7:
        monday = []
        a = df_new[df_new.day == x]
        i = 0
        plt.figure(1, figsize=(50, 3))

        while i < 24:
            b = a[a.hour == i]
            c = b[pollutant].mean()
            i = i + 1
            monday.append(c)
        df_1 = pd.DataFrame(monday)
        df_1.columns = [pollutant]
        # plt.figure(1)
        plt.subplot(1, 7, sub)
        plt.subplots_adjust(wspace=0.2)
        sub = sub + 1
        a = df_1[pollutant].plot.line(color="#ff9999")
        # a.axes.get_xaxis().set_visible(False)
        a.yaxis.set_label_position("left")
        plt.ylabel(pollutant)
        plt.xlabel("hours")
        plt.ylim((20, 70))
        plt.xlim((0, 23))
        plt.title(dayWeek[x])
        plt.grid()

        x = x + 1
    # plot 2
    a = df_new
    hourly = []
    i = 0
    plt.figure(2, figsize=(30, 3))
    while i < 24:
        b = a[a.hour == i]
        c = b[pollutant].mean()
        i = i + 1
        hourly.append(c)
    df_1 = pd.DataFrame(hourly)
    df_1.columns = [pollutant]
    plt.subplot(1, 3, 1)
    a = df_1[pollutant].plot.line(color="#ff9999")
    a.yaxis.set_label_position("left")
    plt.ylabel(pollutant)
    plt.xlabel("hour")
    plt.ylim((20, 45))
    plt.xlim((0, 23))
    plt.grid()

    # plot 3
    a = df_new.fillna(method="ffill")
    monthly = []
    i = 0
    plt.figure(2, figsize=(30, 3))
    while i < 12:
        b = a[a.month == i]
        c = b[pollutant].mean()
        i = i + 1
        monthly.append(c)
    df_1 = pd.DataFrame(monthly)
    df_1.columns = [pollutant]
    plt.subplot(1, 3, 2)
    a = df_1[pollutant].plot.line(color="#ff9999")
    a.yaxis.set_label_position("left")
    plt.ylabel(pollutant)
    plt.xlabel("month")
    plt.ylim((30, 40))
    plt.xlim((0, 12))
    plt.grid()

    # plot 4
    x = 0
    weekday = []
    while x < 7:
        monday = []
        a = df_new[df_new.day == x]
        i = 0
        plt.figure(2, figsize=(30, 3))

        while i < 24:
            b = a[a.hour == i]
            c = b[pollutant].mean()
            i = i + 1
            monday.append(c)
        df_1 = pd.DataFrame(monday)
        df_1.columns = [pollutant]
        weekday.append(df_1.mean())
        x = x + 1
    # print(weekday)
    weekday = pd.DataFrame(weekday)
    weekday.columns = [pollutant]
    plt.subplot(1, 3, 3)
    a = weekday[pollutant].plot.line(color="#ff9999")
    a.yaxis.set_label_position("left")
    plt.ylabel(pollutant)
    plt.xlabel("weekday")
    plt.ylim((26, 40))
    plt.xlim((0, 6))
    plt.grid()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# timeVariation(df,'pm10')
# =============================================================================



def googleMaps(df, lat, long, pollutant, dataLoc):
    """ Plots a folium plot of longitude and latitude points
        provided with information about each point when clicked 
        on
		
		Parameters
		----------
		df: data frame
			minimally containing date and values of pollutant, city,
            longitude, latitude, and AQI
		lat: type string
			Name of column in df of where latitude points are
        long: type string
			Name of column in df of where longitude points are
        pollutant: type string
            Name of pollutant 
        dataLoc: type string
            Name of df column where pollutanat values are stored
    """
    import folium
    import webbrowser
    import pandas as pd
    import matplotlib.pyplot as plt
    import numpy as np
    import pandas as pd

    latitude = 37.0902
    longitude = -95.7129
    Arithmetic_Mean_map = folium.Map(location=[latitude, longitude], zoom_start=4)

    # =============================================================================
    # df = pd.read_csv('interpolData.csv')
    # =============================================================================

    some_value = pollutant
    df = df.loc[df["Parameter Name"] == some_value]

    some_value = "2018-05-07"
    df = df.loc[df["Date Local"] == some_value]

    df = df.sample(frac=1)

    # df_train, df_test = train_test_split(df, test_size=0.2)
    df["Arithmetic Mean Q"] = pd.qcut(df[dataLoc], 4, labels=False)
    colordict = {0: "lightblue", 1: "lightgreen", 2: "orange", 3: "red"}

    for lat, lon, Arithmetic_Mean_Q, Arithmetic_Mean, city, AQI in zip(
        df[lat],
        df[long],
        df["Arithmetic Mean Q"],
        df[dataLoc],
        df["City Name"],
        df["AQI"],
    ):
        folium.CircleMarker(
            [lat, lon],
            radius=0.15 * AQI,
            popup=(
                "City: "
                + str(city).capitalize()
                + "<br>"
                #'Bike score: ' + str(bike) + '<br>'
                "Arithmetic_Mean level: "
                + str(Arithmetic_Mean)
                + "%"
            ),
            color="b",
            key_on=Arithmetic_Mean_Q,
            threshold_scale=[0, 1, 2, 3],
            fill_color=colordict[Arithmetic_Mean_Q],
            fill=True,
            fill_opacity=0.7,
        ).add_to(Arithmetic_Mean_map)
    Arithmetic_Mean_map.save("mymap.html")


# df = pd.read_csv('interpolData.csv')
# googleMaps(df,'Latitude','Longitude','Ozone','Arithmetic Mean')



# -*- coding: utf-8 -*-
"""
Created on Fri Jul 26 02:20:37 2019

@author: Man Vinayaka
"""

# -*- coding: utf-8 -*-
"""
Created on Fri Jul 26 01:59:09 2019

@author: Man Vinayaka
"""


def smoothTrend(df, pollutant, Type):
    """ Plots a connected scatter plot of the average value of
        the pollutant every month of every year. Then plots a
        smooth line of best fit through the plot showing the user
        the overall trend of the pollutant through the years.

        Parameters
        ----------
        df: data frame
            minimally containing date and at least one other
            pollutant 
        pollutant: type string
            A pollutant name correspoinding to 
            a variable in a data frame, ex: 'pm25'
        Type: type int
            Value can be either 1 or 2
            1: normal trend line
            2: bootstrap uncertainties
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import seaborn as sns
    import scipy
    from scipy import stats
    import math
    from scipy.ndimage.filters import gaussian_filter1d

    # =============================================================================
    # df = pd.read_csv("mydata.csv")
    # =============================================================================
    df.index = pd.to_datetime(df.date)
    unique_years = np.unique(df.index.year)
    # df = df[pd.notnull(df[pollutant])]

    i = 0
    year = []
    while i < len(unique_years):
        year.append(str(unique_years[i]))
        i = i + 1
    num_unique_years = len(year)

    i = 0
    values = []
    while i < num_unique_years:
        df_v = df[year[i]].resample("1D").mean()
        df_v = df_v.fillna(method="ffill")
        df_v["month"] = df_v.index.month
        # df_new.index.dayofweek
        nox = df_v[pollutant].mean()
        values.append(nox)
        i = i + 1
    values = gaussian_filter1d(values, sigma=0.75)
    # df = df.drop("date", axis=1)
    # print(df)
    i = 0
    x = 0
    j = 0
    var2 = []
    while i < num_unique_years:
        df_new = df[year[j]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        # df_new['day']=df_new.index.dayofweek
        # df_new['hour']=df_new.index.hour
        i = i + 1
        j = j + 1
        x = 0
        while x < 12:
            a = df_new[df_new.month == x]
            mean_var2 = a[pollutant].mean()
            var2.append(mean_var2)
            x = x + 1
    i = 0
    while i < len(var2):
        if pd.notnull(var2[i]) == False:
            var2[i] = (var2[i - 1] + var2[i + 1]) / 2
        i = i + 1

    scatterX = []
    t = 0
    while t < num_unique_years:
        r = 0
        while r < 12:
            scatterX.append(t + (r / 12))
            r = r + 1
        t = t + 1

    y = var2
    x = scatterX

    def best_fit(X, Y):

        xbar = sum(X) / len(X)
        ybar = sum(Y) / len(Y)
        n = len(X)  # or len(Y)

        numer = sum([xi * yi for xi, yi in zip(X, Y)]) - n * xbar * ybar
        denum = sum([xi ** 2 for xi in X]) - n * xbar ** 2

        b = numer / denum
        a = ybar - b * xbar

        # print('best fit line:\ny = {:.2f} + {:.2f}x'.format(a, b))

        return a, b

    a, b = best_fit(x, y)

    fig = plt.figure()
    ax = fig.add_subplot(111)
    # print(len(x))
    ax.plot(x, y, "-o", color="red")
    ax.set_xlabel("Year")
    ax.set_ylabel("concentration")
    ax.set_title("monthly mean " + pollutant)
    if Type == 1:
        plt.plot(
            np.unique(x), np.poly1d(np.polyfit(x, y, 1))(np.unique(x)), color="black"
        )
    else:
        plt.plot(values, color="black")
    plt.show()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# smoothTrend(df, 'o3',1)
# =============================================================================



# -*- coding: utf-8 -*-
"""
Created on Sun Jul 21 21:50:55 2019

@author: Man Vinayaka
"""


def timePlot(df, year, month):
    """ Plot time series of ws,nox,o3,pm25,pm10 given a month and year
		
		Parameters
		----------
		df: data frame
			a data frame of time series. Must have a date field
			and at least one variable to plot
		year: type string
			year of which data will be cut
		month: type int
			month of what plot will be graphed
    """
    import numpy as np
    import pandas as pd
    import matplotlib as mpl
    import matplotlib.pyplot as plt

    # Cuts the df down to the month specified
    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_n = df[year]
    df_n = df_n.fillna(method="ffill")
    df_n["month"] = df_n.index.month
    df_n.index.dayofweek
    df_n_1 = df_n[df_n.month == month]
    # New lists that have the value of the pollutant in the month specified
    ws = df_n_1["ws"]
    nox = df_n_1["nox"]
    o3 = df_n_1["o3"]
    pm25 = df_n_1["pm25"]
    pm10 = df_n_1["pm10"]

    plt.figure(1)
    # series of 5 plots in one large plot that contains the
    # time series of the polutants

    plt.subplot(511)
    a = nox.plot.line(color="red")
    a.axes.get_xaxis().set_visible(False)
    a.yaxis.set_label_position("right")
    plt.ylabel("nox")

    plt.subplot(512)
    a = o3.plot.line(color="blue")
    a.axes.get_xaxis().set_visible(False)
    a.yaxis.set_label_position("right")
    plt.ylabel("o3")

    plt.subplot(513)
    a = pm25.plot.line(color="green")
    a.axes.get_xaxis().set_visible(False)
    a.yaxis.set_label_position("right")
    plt.ylabel("pm25")

    plt.subplot(514)
    a = pm10.plot.line(color="purple")
    a.axes.get_xaxis().set_visible(False)
    a.yaxis.set_label_position("right")
    plt.ylabel("pm10")

    plt.subplot(515)
    a = ws.plot.line(color="orange")
    a.yaxis.set_label_position("right")
    plt.ylabel("ws")


# =============================================================================
# mydata = pd.read_csv('mydata.csv')
# timePlot(mydata,'2003',8)
# =============================================================================



# #! /usr/bin/env python
# # -*- coding: utf-8 -*-
# """
#     Setup file for vayu.
#     Use setup.cfg to configure your project.

#     This file was generated with PyScaffold 3.2.1.
#     PyScaffold helps you to put up the scaffold of your new Python project.
#     Learn more under: https://pyscaffold.org/
# """
# import sys

# from pkg_resources import VersionConflict, require
# from setuptools import setup

# try:
#     require("setuptools>=38.3")
# except VersionConflict:
#     print("Error: version of setuptools is too old (<38.3)!")
#     sys.exit(1)


# if __name__ == "__main__":
#     setup(use_pyscaffold=True)
#! /usr/bin/env python
"""A template for scikit-learn compatible packages."""


import codecs
import os

from setuptools import find_packages, setup

# get __version__ from _version.py
ver_file = os.path.join('vayu', '_version.py')
with open(ver_file) as f:
    exec(f.read())

DISTNAME = 'vayu'
DESCRIPTION = 'Tools for Air Quality Data Analysis.'
with codecs.open('README.md', encoding='utf-8-sig') as f:
    LONG_DESCRIPTION = f.read()
MAINTAINER = 'Sustainability-Lab@IITGN'
MAINTAINER_EMAIL = 'apoorv.agnihotri@iitgn.ac.in, deepak.narayanan@iitgn.ac.in'
URL = 'https://sustainability-lab.github.io/vayu'
LICENSE = 'MIT'
DOWNLOAD_URL = 'https://sustainability-lab.github.io/vayu'
VERSION = __version__
INSTALL_REQUIRES = [
    'matplotlib', 'numpy', 
    'pandas', 'geopandas', 'scipy', 
    'scikit-learn' 
    ]
CLASSIFIERS = ['Intended Audience :: Science/Research',
               'Intended Audience :: Developers',
               'License :: OSI Approved',
               'Programming Language :: Python',
               'Topic :: Software Development',
               'Topic :: Scientific/Engineering',
               'Operating System :: Microsoft :: Windows',
               'Operating System :: POSIX',
               'Operating System :: Unix',
               'Operating System :: MacOS',
               'Programming Language :: Python :: 3.6',
               'Programming Language :: Python :: 3.7']
EXTRAS_REQUIRE = {
    'tests': [
        'pytest',
        'pytest-cov'],
    'docs': [
        'sphinx',
        'sphinx-gallery',
        'sphinx_rtd_theme',
        'numpydoc',
        'matplotlib'
    ]
}

setup(name=DISTNAME,
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      description=DESCRIPTION,
      license=LICENSE,
      url=URL,
      version=VERSION,
      download_url=DOWNLOAD_URL,
      long_description=LONG_DESCRIPTION,
      zip_safe=False,  # the package can run out of an .egg file
      classifiers=CLASSIFIERS,
      packages=find_packages(),
      install_requires=INSTALL_REQUIRES,
      extras_require=EXTRAS_REQUIRE,
      python_requires='>=3.6')



# import setuptools
# import codecs

# with codecs.open('README.md', encoding='utf-8-sig') as f:
#     long_description = f.read()

# # with open("README.md", "r") as fh:
# #     long_description = fh.read()

# setuptools.setup(
#     name="vayu", # Replace with your own username
#     version="0.0.12",
#     author="Sustainability-Lab@IITGN",
#     author_email="apoorv.agnihotri@iitgn.ac.in",
#     description="Tools for Air Quality Data Analysis",
#     long_description=long_description,
#     long_description_content_type="text/markdown",
#     url="https://sustainability-lab.github.io/vayu/",
#     packages=setuptools.find_packages(),
#     classifiers=[
#         "Programming Language :: Python :: 3",
#         "License :: OSI Approved :: MIT License",
#         "Operating System :: OS Independent",
#     ],
#     setup_requires=['wheel'],
#     install_requires=[
#         'matplotlib', 'numpy', 
#         'pandas', 'geopandas', 'scipy', 
#         'scikit-learn'],
#     python_requires='>=3.6',
# )


# -*- coding: utf-8 -*-
"""
This is a skeleton file that can serve as a starting point for a Python
console script. To run this script uncomment the following lines in the
[options.entry_points] section in setup.cfg:

    console_scripts =
         fibonacci = airPy.skeleton:run

Then run `python setup.py install` which will install the command `fibonacci`
inside your current environment.
Besides console scripts, the header (i.e. until _logger...) of this file can
also be used as template for Python modules.

Note: This skeleton file can be safely removed if not needed!
"""

import argparse
import sys
import logging

from airPy import __version__

__author__ = "manvinayaka"
__copyright__ = "manvinayaka"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def fib(n):
    """Fibonacci example function

    Args:
      n (int): integer

    Returns:
      int: n-th Fibonacci number
    """
    assert n > 0
    a, b = 1, 1
    for i in range(n - 1):
        a, b = b, a + b
    return a


def parse_args(args):
    """Parse command line parameters

    Args:
      args ([str]): command line parameters as list of strings

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser(description="Just a Fibonacci demonstration")
    parser.add_argument(
        "--version", action="version", version="airPy {ver}".format(ver=__version__)
    )
    parser.add_argument(dest="n", help="n-th Fibonacci number", type=int, metavar="INT")
    parser.add_argument(
        "-v",
        "--verbose",
        dest="loglevel",
        help="set loglevel to INFO",
        action="store_const",
        const=logging.INFO,
    )
    parser.add_argument(
        "-vv",
        "--very-verbose",
        dest="loglevel",
        help="set loglevel to DEBUG",
        action="store_const",
        const=logging.DEBUG,
    )
    return parser.parse_args(args)


def setup_logging(loglevel):
    """Setup basic logging

    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(
        level=loglevel, stream=sys.stdout, format=logformat, datefmt="%Y-%m-%d %H:%M:%S"
    )


def main(args):
    """Main entry point allowing external calls

    Args:
      args ([str]): command line parameter list
    """
    args = parse_args(args)
    setup_logging(args.loglevel)
    _logger.debug("Starting crazy calculations...")
    print("The {}-th Fibonacci number is {}".format(args.n, fib(args.n)))
    _logger.info("Script ends here")


def run():
    """Entry point for console_scripts
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    run()
